# auto-generated by dev/info-updater.py
# using metadata from dev/metadata.py

SETUP_INFO = dict(
    name = 'walt-common',
    version = '4',
    install_requires = ['plumbum>=1.4.2'],
    author = 'WalT developers',
    author_email = 'walt-contact@univ-grenoble-alpes.fr',
    keywords = 'WalT testbed',
    license = '3-Clause BSD',
    url = 'https://walt-project.liglab.fr',
    description = 'WalT common python modules.'
)
